var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Api } from "../../Api";
import { TypedEvent } from "../../TypedEvent";
import { FieldChangeResultType, FieldChangeResult } from "../FieldChangeResult";
import { DevicesService } from "./DevicesService";
import { ChannelConfigurations } from "./IDeviceChannel";
import { DeviceTaskExecutionResult } from "./TaskExecutionResult";
import { ExalusConnectionService } from "../ExalusConnectionService";
import { DataFrame, Method, Status } from "../../DataFrame";
import { DependencyContainer } from '../../DependencyContainer';
import { Device } from "./Device";
export class DeviceChannel {
    constructor() {
        this._device = new Device();
        this._number = 0;
        this._name = "";
        this._iconName = "";
        this._states = [];
        this._mode = ChannelConfigurations.Default;
        this._channelGroups = [];
        this._isHidden = false;
        this._availableTasksTypes = [];
        this._availableResponseTypes = [];
        this._onTasksExecutionChangedEvent = new TypedEvent();
    }
    SetDevice(device) {
        this._device = device;
        this._device.OnDeviceStateChangedEvent().Subscribe(newState => {
            const channel = newState.Data.Channel;
            if (channel != this._number)
                return;
            this._states = this._states.where(state => state.TypeAsEnum != newState.TypeAsEnum).toArray();
            this._states.push(newState);
        });
    }
    GetDevice() {
        return this._device;
    }
    GetChannelConfigurationData() {
        const data = new ChannelConfigurationData();
        data.Channel = this._number;
        data.ChannelName = this._name;
        data.Configuration = this._mode;
        data.GroupsGuids = this._channelGroups;
        data.Hidden = this._isHidden;
        data.IconName = this._iconName;
        return data;
    }
    HideAsync() {
        const data = this.GetChannelConfigurationData();
        data.Hidden = true;
        return this.ChangeConfigurationAsync(data);
    }
    ShowAsync() {
        const data = this.GetChannelConfigurationData();
        data.Hidden = false;
        return this.ChangeConfigurationAsync(data);
    }
    ChangeNameAsync(name) {
        const data = this.GetChannelConfigurationData();
        data.ChannelName = name;
        return this.ChangeConfigurationAsync(data);
    }
    ChangeIconNameAsync(name) {
        const data = this.GetChannelConfigurationData();
        data.IconName = name;
        return this.ChangeConfigurationAsync(data);
    }
    ChangeGroupsAsync(groups) {
        const data = this.GetChannelConfigurationData();
        data.GroupsGuids = groups;
        return this.ChangeConfigurationAsync(data);
    }
    ChangeConfigurationAsync(configuration) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield Api.Get(ExalusConnectionService.ServiceName)
                    .SendAndWaitForResponseAsync(new DeviceChannelConfigurationRequest(configuration), 20000, false);
                switch (result.Status) {
                    case Status.OK:
                        return new FieldChangeResult(FieldChangeResultType.Changed, "");
                    case Status.WrongData:
                        switch (result.Data) {
                            case "DeviceNotFound":
                                (_a = DependencyContainer.Log) === null || _a === void 0 ? void 0 : _a.Error(`Failed to change channel configuration, referenced device does not exist!`);
                                return new FieldChangeResult(FieldChangeResultType.WrongData, "DeviceNotFound");
                        }
                        break;
                    case Status.FatalError:
                        return new FieldChangeResult(FieldChangeResultType.UnknownError, "");
                }
            }
            catch (ex) {
                if (ex.message.includes("timeout"))
                    return new FieldChangeResult(FieldChangeResultType.ControllerResponseTimeout, "");
            }
            return new FieldChangeResult(FieldChangeResultType.Unknown, "");
        });
    }
    get Number() {
        return this._number;
    }
    set Number(value) {
        this._number = value;
    }
    get Name() {
        return this._name;
    }
    set Name(value) {
        this._name = value;
    }
    get IconName() {
        return this._iconName;
    }
    set IconName(value) {
        this._iconName = value;
    }
    get States() {
        return this._states;
    }
    get Mode() {
        return this._mode;
    }
    set Mode(value) {
        this._mode = value;
    }
    get ChannelGroups() {
        return this._channelGroups;
    }
    set ChannelGroups(value) {
        this._channelGroups = value;
    }
    get IsHidden() {
        return this._isHidden;
    }
    set IsHidden(value) {
        this._isHidden = value;
    }
    get AvailableTaskTypes() {
        return this._availableTasksTypes;
    }
    set AvailableTaskTypes(value) {
        this._availableTasksTypes = value;
    }
    get AvailableResponseTypes() {
        return this._availableResponseTypes;
    }
    set AvailableResponseTypes(value) {
        this._availableResponseTypes = value;
    }
    OnTasksExecutionChangeEvent() {
        return this._onTasksExecutionChangedEvent;
    }
    ExecuteTaskAsync(task) {
        if (this._device != null) {
            task.Channel = this.Number;
            task.DeviceGuid = this._device.Guid;
            return Api.Get(DevicesService.ServiceName).ExecuteDeviceTaskAsync(this._device, task);
        }
        else
            return Promise.resolve(DeviceTaskExecutionResult.Failed);
    }
}
export class ChannelConfigurationData {
    constructor() {
        this.Channel = 0;
        this.ChannelName = "";
        this.Configuration = ChannelConfigurations.Default;
        this.GroupsGuids = [];
        this.IconName = "";
        this.Hidden = false;
    }
}
class DeviceChannelConfigurationRequest extends DataFrame {
    constructor(configuration) {
        super();
        this.Resource = "/devices/device/channel/configuration";
        this.Method = Method.Put;
        this.Data = configuration;
    }
}
//# sourceMappingURL=DeviceChannel.js.map